<?php

class RecordBookData {
    /** @var integer */
    public $id;
    /** @var string */
    public $externalID;
    /** @var integer */
    public $planID;
    /** @var string */
    public $planExternalID;
    /** @var integer */
    public $studentID;
    /** @var string */
    public $studentExternalID;
    /** @var integer */
    public $facultyID;
    /** @var string */
    public $facultyExternalID;
    /** @var string */
    public $status;
    /** @var string */
    public $degree;
    /** @var string */
    public $form;
    /** @var string */
    public $speciality;
    /** @var integer */
    public $grade;
    /** @var integer */
    public $group;
    /** @var integer */
    public $groupID;
}

class StudentData {
    /** @var integer */
    public $id;
    /** @var string */
    public $externalID;
    /** @var string */
    public $firstName;
    /** @var string */
    public $secondName;
    /** @var string */
    public $lastName;
    /** @var RecordBookData[] */
    public $recordBooks;
}

class Controller_Api_V0_Student extends Controller_Handler_Api {
    /** Нормализация данных зачетки
     * @param RecordBookData $recordBookData
     * @return RecordBookData
     */
    private function normalizeRecordBookData($recordBookData) {
        if (!isset($recordBookData->id) && isset($recordBookData->externalID)) {
            $recordBookData->id = Model_RecordBook::withExternalID($recordBookData->externalID);
        }

        if (!isset($recordBookData->studentID)) {
            if (isset($recordBookData->studentExternalID)) {
                $recordBookData->studentID = Model_Student::withExternalID($recordBookData->studentExternalID);
                if ($recordBookData->studentID === -1) {
                    throw new InvalidArgumentException('RecordBook student not found');
                }
            } else {
                throw new InvalidArgumentException('RecordBook without student');
            }
        }

        if (!isset($recordBookData->planID)) {
            if (isset($recordBookData->planExternalID)) {
                $recordBookData->planID = Model_Plan::withExternalID($recordBookData->planExternalID);
                if ($recordBookData->planID == -1) {
                    throw new InvalidArgumentException('RecordBook plan not found');
                }
            } else {
                throw new InvalidArgumentException('RecordBook without plan');
            }
        }

        if (!isset($recordBookData->facultyID)) {
            if (isset($recordBookData->facultyExternalID)) {
                $facultyID = $this->getFacultyIDByExternalID($recordBookData->facultyExternalID);
                if (is_null($facultyID)) {
                    throw new InvalidArgumentException('RecordBook faculty not found');
                }
                $recordBookData->facultyID = $facultyID;
            } else {
                throw new InvalidArgumentException('RecordBook without faculty');
            }
        }

        switch ($recordBookData->degree) {
            case 'Академический бакалавр':
            case 'Бакалавр':
                $recordBookData->degree = 'bachelor';
                break;

            case 'Специалист':
                $recordBookData->degree = 'specialist';
                break;

            case 'Магистр':
                $recordBookData->degree = 'master';
                break;

            case 'Аспирант':
//                $data->degree = 'postgraduate';
//                break;
                throw new InvalidArgumentException('Bad recordBook degree');

            default:
                throw new InvalidArgumentException('Bad recordBook degree');
        }

        switch ($recordBookData->status) {
            case 'Учится':
                $recordBookData->status = 'common';
                break;

            case 'Академ':
                $recordBookData->status = 'outlet';
                break;

            case 'Выпустился':
//                $data->status = 'expulsion';
//                break;
                throw new InvalidArgumentException('Bad recordBook status');

            case 'Отчислен':
//                $data->status = 'leave';
//                break;
                throw new InvalidArgumentException('Bad recordBook status');

            default:
                throw new InvalidArgumentException('Bad recordBook status');
        }

        switch ($recordBookData->form) {
            case 'Очная':
                $recordBookData->form = '1';
                break;

            case 'Очно-заочная':
//                $data->form = '2';
//                break;
                throw new InvalidArgumentException('Bad recordBook form');

            case 'Заочная':
//                $data->form = '3';
//                break;
                throw new InvalidArgumentException('Bad recordBook form');

            default:
                throw new InvalidArgumentException('Bad recordBook form');
        }

        if (!isset($recordBookData->groupID)) {
            if (isset($recordBookData->grade) && isset($recordBookData->degree)) {
                $gradeID = Model_Grades::getID($recordBookData->grade, $recordBookData->degree);
                if ($gradeID == -1) {
                    throw new InvalidArgumentException('RecordBook grade not found');
                }

                $group = Model_Group::find($gradeID, $recordBookData->group, $recordBookData->facultyID);
                if (is_null($group)) {
                    throw new InvalidArgumentException('RecordBook group not found');
                }

                $recordBookData->groupID = $group->getID();
            } else {
                throw new InvalidArgumentException('RecordBook without grade or grade');
            }
        }

        return $recordBookData;
    }

    /** Нормализация данных студента
     * @param StudentData $studentData
     * @return StudentData
     */
    private function normalizeStudentData($studentData, $facultyID) {
        if (!isset($studentData->id) && isset($studentData->externalID)) {
            $studentData->id = Model_Student::withExternalID($studentData->externalID);
        }

        if (!isset($studentData->id) || ($studentData->id == -1)) {
            $foundList = Model_Students::searchStudentsByName([$studentData->firstName, $studentData->secondName, $studentData->lastName], $facultyID);
            $foundSet = [];
            foreach ($foundList as $item) {
                if (!in_array($item['ID'], $foundSet)) {
                    $foundSet[] = $item['ID'];
                }
            }
            if (count($foundSet) === 1) {
                $studentData->id = (int)$foundList[0];
            } elseif (count($foundList) > 1) {
                throw new InvalidArgumentException('Duplicate student');
            }
        }

        foreach ($studentData->recordBooks as &$recordBook) {
            if (isset($studentData->id)) {
                $recordBook->studentID = $studentData->id;
            }

            if (isset($studentData->externalID)) {
                $recordBook->studentExternalID = $studentData->externalID;
            }

            try {
                $recordBook = $this->normalizeRecordBookData($recordBook);
            } catch (Exception $e) {
                Log::instance()->add(Log::WARNING, '{0} {1}: {2}', array(
                    '{0}' => 'SYNC_STUDENTS',
                    '{1}' => $e->getMessage(),
                    '{2}' => json_encode($recordBook),
                ));
                $recordBook = null;
            }
        }

        return $studentData;
    }

    /**
     * @param RecordBookData $recordBookData
     * @return integer
     */
    private function createOrUpdateRecordBook($recordBookData) {
        $recordBookBuilder = Model_RecordBook::make()
            ->studentID($recordBookData->studentID)
            ->planID($recordBookData->planID)
            ->semesterID(Model_System::$SemesterID) // todo wtf
            ->group($recordBookData->groupID)
            ->externalID($recordBookData->externalID);

        if (isset($recordBookData->id) && $recordBookData->id > 0) {
            $recordBook = Model_RecordBook::load($recordBookData->id);
            $recordBook->update($recordBookBuilder->getPreparedData());
        } else {
            $recordBook = $recordBookBuilder->create();
        }

        return (int)$recordBook->ID;
    }

    /**
     * @param StudentData $studentData
     * @return integer[]
     */
    private function createOrUpdateStudent($studentData) {
        if (isset($studentData->id) && $studentData->id > 0) {
            $student = Model_Student::load($studentData->id);
        } else {
            $student = Model_Student::make()
                ->externalID($studentData->externalID)
                ->firstName($studentData->firstName)
                ->secondName($studentData->secondName)
                ->lastName($studentData->lastName)
                ->create();
        }

        $recordBookIDs = [];

        foreach ($studentData->recordBooks as $recordBookData) {
            if (is_null($recordBookData)) continue;

            $recordBookData->studentID = $student->ID;

            try {
                $recordBookIDs[] = $this->createOrUpdateRecordBook($recordBookData);
            } catch (Exception $e) {
                Log::instance()->add(Log::WARNING, '{0} {1}: {2}', array(
                    '{0}' => 'SYNC_STUDENTS',
                    '{1}' => $e->getMessage(),
                    '{2}' => json_encode($recordBookData),
                ));
            }
        }

        return $recordBookIDs;
    }

//    public function action_get_index() {
//        $this->response->body('{ what: get index }');
//
//        return '{ what: get index }'; //$this->response;
//    }

//    /**
//     * @api {put} api/v0/student/recordBook Add new record book(s)
//     * @apiName Add new record book
//     * @apiGroup Record books
//     * @apiVersion 0.1.2
//     * @apiParam {String} token Api key
//     * @apiParam {String} [batch] Flag about massive addition
//     * @apiParam {String} externalID
//     * @apiParam {String} studentExternalID
//     * @apiParam {String} planExternalID
//     * @apiParam {String} facultyExternalID
//     * @apiParam {String} status
//     * @apiParam {String} level
//     * @apiParam {String} form
//     * @apiParam {String} speciality
//     * @apiParam {Int} grade
//     * @apiParam {Int} group
//     */
//    public function action_put_recordBook() {
//        try {
//            if ($this->request->query('batch') !== NULL) {
//                $data = json_decode($this->request->body());
//
//                $res = [];
//                foreach ($data as $item) {
//                    $data = $this->normalizeRecordBookData($item);
//                    $res[] = $this->createRecordBook($data);
//                }
//            } else {
//                /** @var RecordBookData $data */
//                $data = (object)$this->request->query();
//
//                $data = $this->normalizeRecordBookData($data);
//                $res[] = $this->createRecordBook($data);
//            }
//        } catch (Exception $e) {
//            $this->badRequestError($e->getMessage());
//        }
//
//        return $res;
//    }

//    /**
//     * @api {put} api/v0/student Add new student(s)
//     * @apiName Add new student
//     * @apiGroup Student
//     * @apiVersion 0.1.0
//     * @apiParam {String} token Api key
//     * @apiParam {String} [batch] Flag about massive addition
//     * @apiParam {String} firstName
//     * @apiParam {String} secondName
//     * @apiParam {String} lastName
//     * @apiParam {String} departmentName
//     * @apiParam {String} [departmentID]
//     * @apiParam {String} jobPositionName
//     * @apiParam {String} [jobPositionID]
//     * @apiParam {String} status
//     */
    public function action_put_index() {
        $res = [];
        try {
            if ($this->request->query('batch') !== NULL) {
                $data = json_decode($this->request->body());
                $facultyID = Model_Faculties::getIdByExternalID($data->faculty);

//                foreach ($data->students as $student) {
//                    try {
//                        $studentData = $this->normalizeStudentData($student, $facultyID);
//                        $res[] = $this->createOrUpdateStudent($studentData);
//                    } catch (Exception $e) {
//                        Log::instance()->add(Log::WARNING, '{0} {1}: {2}', array(
//                            '{0}' => 'SYNC_STUDENTS',
//                            '{1}' => $e->getMessage(),
//                            '{2}' => json_encode($student),
//                        ));
//                    }
//                }

                try {
                    $student = $data->students[2];
                    $res[] = $student;
                    $studentData = $this->normalizeStudentData($student, $facultyID);
                    $res[] = $this->createOrUpdateStudent($studentData);
                } catch (Exception $e) {
                    Log::instance()->add(Log::WARNING, '{0} {1}: {2}', array(
                        '{0}' => 'SYNC_STUDENTS',
                        '{1}' => $e->getMessage(),
                        '{2}' => json_encode($student),
                    ));
                }
            } else {
//                $data = (object)$this->request->query();
//
//                $studentData = $this->normalizeStudentData($data);
//                $res[] = $this->createOrUpdateStudent($studentData);
                throw new InvalidArgumentException('No batch data received!');
            }
        } catch (Exception $e) {
            $this->badRequestError($e->getMessage());
        }

        return $res;
    }

//    public function action_get_list_by_name() {
//        return '{ what: nothing }';
//    }
}
